from geodesic_chenhan import (
    CPoint3D,
    CFace,
    CBaseModel,
    CRichModel,
    CICHWithFurtherPriorityQueue,
)


def rotate(l, n):
    return l[n:] + l[:n]


def test():
    points = [CPoint3D(0.0, 0.0, 0.0), CPoint3D(0.5, 1.0, 0.0), CPoint3D(1.0, 0.0, 0.0)]
    faces = [CFace(0, 1, 2)]
    bmodel = CBaseModel()
    rmodel = CRichModel()
    bmodel.LoadModel(points, faces)
    print(bmodel.GetNumOfVerts())

    rmodel.LoadModel(points, faces)
    rmodel.Preprocess()
    print(rmodel.GetNumOfVerts())

    emethod = CICHWithFurtherPriorityQueue(rmodel, set([0]))
    emethod.Execute()

    print("RUN TIMER ::: ", emethod.GetRunTime())
    print("MEMORY COST ::: ", emethod.GetMemoryCost())
    print("GET TOTAL WINDOWS CREATED ::: ", emethod.GetWindowNum())
    print("LENGTH OF THE QUEUE ::: ", emethod.GetMaxLenOfQue())
    print("WHAT IS NPE BTW? ::: ", emethod.GetNPE())
    print("Get Sequence TREE DEPTH ::: ", emethod.GetDepthOfSequenceTree())
    print("HAS ALGORITHM COMPLETED ITS WORK?::: ", emethod.HasBeenCompleted())
    print(emethod.HasBeenCompleted())
    print([iav.disUptodate for iav in emethod.GetVertexDistances()])

    paths = emethod.FindSourceVertex(2, [])
    paths = rotate(paths, 1)
    for epoint in paths:
        pt = epoint.Get3DPoint(rmodel)
        print(
            "PATH POINTS :::",
            pt.x,
            pt.y,
            pt.z,
            epoint.leftVertIndex,
            epoint.rightVertIndex,
            epoint.isVertex,
        )


if __name__ == "__main__":
    test()

    path_verts = []



import os
import site
import sys
import pkg_resources
from setuptools import setup, Extension
from Cython.Distutils import build_ext
from autowrap.Main import run as autowrap_run

# This line enable user based installations when in editable mode
site.ENABLE_USER_SITE = "--user" in sys.argv[1:]

data_dir = pkg_resources.resource_filename("autowrap", "data_files")
include_dir = os.path.join(data_dir, "autowrap")

autowrap_run(
    [os.path.abspath('./src/chenhancc.pxd')],
    [], [], os.path.abspath('./src/geodesic_chenhan.pyx'))

ext = Extension(
    "geodesic_chenhan",
    sources=['src/geodesic_chenhan.cpp'],
    language="c++",
    extra_compile_args=["-std=c++14"],
    extra_link_args=["-std=c++14"],
    include_dirs=[include_dir, data_dir],
)

setup(
    cmdclass={'build_ext': build_ext},
    ext_modules=[ext],
)



from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from libcpp.set cimport set as libcpp_set

cdef extern from "Point3D.hpp":        
    cdef cppclass CPoint3D:
        double x
        double y
        double z
        CPoint3D()
        CPoint3D(CPoint3D other)
        CPoint3D(double, double, double)        
        void Normalize()
        double Len()

cdef extern from "BaseModel.hpp": 
    cdef cppclass CFace:
        CFace();
        CFace(int, int, int)
        void setIndices(int, int, int)
    
    cdef cppclass CBaseModel:
        CBaseModel();
        void LoadModel(libcpp_vector[CPoint3D] vertices, libcpp_vector[CFace] faces)
        int GetNumOfVerts();
        int GetNumOfFaces();
        bool isAValidModel();
        CPoint3D Vert(int);

cdef extern from "RichModel.hpp":
    cdef cppclass CEdge:
        int indexOfLeftVert;
        int indexOfRightVert;
        int indexOfOppositeVert;
        int indexOfLeftEdge;
        int indexOfRightEdge;
        int indexOfReverseEdge;
        int indexOfFrontFace;
        double length;      
        CEdge();          
        CEdge(CEdge other)  
    
    cdef cppclass EdgePoint:
        int index;
        int leftVertIndex;
        int rightVertIndex;
        double proportion;
        bool isVertex;
        EdgePoint();
        EdgePoint(EdgePoint other);
        EdgePoint(int);
        EdgePoint(int, double);
        EdgePoint(CRichModel, int, int, double);
        CPoint3D Get3DPoint(CRichModel);
    
    cdef cppclass CRichModel(CBaseModel):
        # wrap-inherits:
        #     CBaseModel
        #        
        CRichModel();
        void Preprocess();
        int GetNumOfEdges();
        int GetNumOfValidDirectedEdges();
        int GetNumOfTotalUndirectedEdges();
        int GetNumOfGenera();
        int GetNumOfIsolated();
        int GetNumOfComponents();
        int GetNumOfBoundries();
        bool IsConvexVert(int);
        bool isBoundaryVert(int);
        bool IsClosedModel();
        bool IsExtremeEdge(int)
        bool IsStartEdge(int);
        bool HasBeenProcessed();
        CEdge Edge(int);
    
        
cdef extern from "ExactMethodForDGP.hpp":
    cdef cppclass InfoAtVertex:
        InfoAtVertex()
        InfoAtVertex(InfoAtVertex other)
        bool fParentIsPseudoSource
        char birthTime
        int indexOfParent
        int indexOfRootVertOfParent
        int level
        double disUptodate
        double entryProp
        
    cdef cppclass CExactMethodForDGP:
        # wrap-ignore    
        # ABSTRACT class
        #
        CExactMethodForDGP(CRichModel inputModel, libcpp_set[int] indexOfSourceVerts)  except +
        int GetRootSourceOfVert(int index)
        libcpp_vector[EdgePoint] FindSourceVertex(int indexOfVert, libcpp_vector[EdgePoint] resultingPath)
        void PickShortestPaths(int num)
        void Execute()        
        double GetRunTime()
        double GetMemoryCost()
        int GetWindowNum()
        int GetMaxLenOfQue()
        double GetNPE()
        int GetDepthOfSequenceTree()
        bool HasBeenCompleted()
        libcpp_vector[InfoAtVertex] GetVertexDistances();

cdef extern from "PreviousCH.hpp":
    cdef cppclass CPreviousCH(CExactMethodForDGP):
        # wrap-inherits:
        #     CExactMethodForDGP
        #
        CPreviousCH(CRichModel inputModel, libcpp_set[int] indexOfSourceVerts)  except +


cdef extern from "ImprovedCHWithEdgeValve.hpp":
    cdef cppclass CImprovedCHWithEdgeValve(CPreviousCH):
        # wrap-inherits:
        #     CPreviousCH
        #
        CImprovedCHWithEdgeValve(CRichModel inputModel, libcpp_set[int] indexOfSourceVerts)  except +

cdef extern from "ICHWithFurtherPriorityQueue.hpp":
    cdef cppclass CICHWithFurtherPriorityQueue(CImprovedCHWithEdgeValve):
        # wrap-inherits:
        #     CImprovedCHWithEdgeValve
        #
        CICHWithFurtherPriorityQueue(CRichModel inputModel, libcpp_set[int] indexOfSourceVerts)  except +    
    
    